/***************************************************************************
 *   Copyright (C) 2005 by Paul Lutus   *
 *   lutusp@pl-alpha   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "controlpanelmanager.h"


ControlPanelManager::ControlPanelManager(RayTracerWidgetCode *p) {
    parent = p;
    enableControlPanel(false);
    inhibit = false;
}


ControlPanelManager::~ControlPanelManager() {}

void ControlPanelManager::enableControlPanel(bool state) {
    parent->controlPanelGroup->setEnabled(state);
    parent->deleteButton->setEnabled(state);
    parent->cloneButton->setEnabled(state);
    if(!state) {
        currentLens = NULL;
    }
}

template <class A>
string ControlPanelManager::formattedNumber(A v) {
    stringstream ss;
    ss << v;
    return ss.str();
}

template <class A>
void ControlPanelManager::loadLineEdit(QLineEdit *le, A v) {
    string s = formattedNumber(v);
    le->setText(s.c_str());
}

void ControlPanelManager::loadCheckBox(QCheckBox *cb,bool state) {
    cb->setChecked(state);
}

void ControlPanelManager::retrieveCheckBox(QCheckBox *cb,bool &state) {
    convertValue(cb->isChecked(),state);
}

template <class A, class B>
void ControlPanelManager::convertValue(A s,B &v) {
    stringstream ss;
    ss << s;
    ss >> v;
}

void ControlPanelManager::retrieveLineEdit(QLineEdit *p,double &v) {
    convertValue(p->text(),v);
}

void ControlPanelManager::updateLensDisplay(Lens *lens) {
    inhibit = true;
    enableControlPanel(true);
    currentLens = lens;
    loadCheckBox(parent->symmetricalCheckBox,lens->symmetrical);

    loadLineEdit(parent->lensRadiusText,lens->lensRadius);
    loadLineEdit(parent->leftRadiusText,lens->leftSphereRadius);
    loadLineEdit(parent->thicknessText,lens->userThickness);
    loadLineEdit(parent->refractionText,lens->ior);
    loadLineEdit(parent->dispersionText,lens->dispersion);
    loadLineEdit(parent->leftCurvatureText,lens->lcf);
    loadLineEdit(parent->xPositionText,lens->cx);
    loadLineEdit(parent->yPositionText,lens->cy);
    loadCheckBox(parent->leftHypCheckBox,lens->leftHyp);

    if(lens->symmetrical) {
        lens->rightSphereRadius = lens->leftSphereRadius;
        lens->rcf = lens->lcf;
        lens->rightHyp = lens->leftHyp;
        loadLineEdit(parent->rightRadiusText,lens->leftSphereRadius);
        loadLineEdit(parent->rightCurvatureText,lens->lcf);
    } else { // not symmetrical
        loadLineEdit(parent->rightRadiusText,lens->rightSphereRadius);
        loadLineEdit(parent->rightCurvatureText,lens->rcf);
        loadCheckBox(parent->rightHypCheckBox,lens->rightHyp);
    }

    parent->leftCurvatureText->setEnabled(lens->leftHyp);
    parent->rightCurvatureText->setEnabled(lens->rightHyp && !lens->symmetrical);
    parent->rightRadiusText->setEnabled(!lens->symmetrical);
    parent->rightHypCheckBox->setEnabled(!lens->symmetrical);

    inhibit = false;
}



void ControlPanelManager::changeLens() {
    if(currentLens != NULL && !inhibit) {
        Lens *lens = currentLens; // just for naming convenience
        retrieveCheckBox(parent->symmetricalCheckBox,lens->symmetrical);

        retrieveLineEdit(parent->leftRadiusText,lens->leftSphereRadius);
        retrieveLineEdit(parent->leftCurvatureText,lens->lcf);
        retrieveCheckBox(parent->leftHypCheckBox,lens->leftHyp);
        retrieveLineEdit(parent->lensRadiusText,lens->lensRadius);
        retrieveLineEdit(parent->thicknessText,lens->userThickness);
        retrieveLineEdit(parent->refractionText,lens->ior);
        retrieveLineEdit(parent->dispersionText,lens->dispersion);
        retrieveLineEdit(parent->xPositionText,lens->cx);
        retrieveLineEdit(parent->yPositionText,lens->cy);

        if(lens->symmetrical) {
            lens->rightSphereRadius = lens->leftSphereRadius;
            lens->rcf = lens->lcf;
            lens->rightHyp = lens->leftHyp;
            loadLineEdit(parent->rightRadiusText,lens->rightSphereRadius);
            loadLineEdit(parent->rightCurvatureText,lens->rcf);
            loadCheckBox(parent->rightHypCheckBox,lens->rightHyp);
        } else { // not symmetrical
            retrieveLineEdit(parent->rightRadiusText,lens->rightSphereRadius);
            retrieveLineEdit(parent->rightCurvatureText,lens->rcf);
            retrieveCheckBox(parent->rightHypCheckBox,lens->rightHyp);
        }

        parent->leftCurvatureText->setEnabled(lens->leftHyp);
        parent->rightCurvatureText->setEnabled(lens->rightHyp && !lens->symmetrical);
        parent->rightRadiusText->setEnabled(!lens->symmetrical);
        parent->rightHypCheckBox->setEnabled(!lens->symmetrical);

        lens->setup();
        parent->tad->rayTraceProcess(true);

    }
}

void ControlPanelManager::newLens() {
    double xDelta = 0;
    if(currentLens != NULL) {
        xDelta = currentLens->cx + 1;
    }
    Lens *nl = new Lens(parent,xDelta,0,2,6,6,0,1.52,.001,.001,59,0,0,1);
    parent->tad->lensVec.push_back(nl);
    currentLens = nl;
    updateLensDisplay(currentLens);
    parent->tad->rayTraceProcess(true);
}

void ControlPanelManager::cloneLens() {
    if(currentLens != NULL) {
        string s = currentLens->describe();
        Lens *nl = new Lens(parent,s);
        nl->cx += 1;
        parent->tad->lensVec.push_back(nl);
        currentLens = nl;
        updateLensDisplay(currentLens);
        parent->boxTarget = currentLens;
        fflush(stdout);
        parent->tad->rayTraceProcess(true);
    }
}
void ControlPanelManager::deleteLens() {
    if(currentLens != NULL) {
        int confirm = QMessageBox::question(
                          parent,
                          "Delete Lens",
                          "Delete current Lens?",
                          QMessageBox::Yes,QMessageBox::No,QMessageBox::NoButton);
        if(confirm == QMessageBox::Yes) {
            VLI vli = parent->tad->lensVec.begin();
            while(vli < parent->tad->lensVec.end()) {
                if (currentLens == (*vli)) {
                    break;
                }
                vli++;
            }
            if(vli != parent->tad->lensVec.end()) {
                parent->tad->lensVec.erase(vli);
                enableControlPanel(false);
                parent->boxTarget = NULL;
            }
        }
    }
    parent->tad->rayTraceProcess(true);
}

void ControlPanelManager::writeGlobalDisplay() {
    loadLineEdit(parent->yStartBeamPosText,parent->yStartBeamPos);
    loadLineEdit(parent->yEndBeamPosText,parent->yEndBeamPos);
    loadLineEdit(parent->dispersionBeamsText,parent->dispersionBeams);
    loadLineEdit(parent->tracingBeamsText,parent->tracingBeams);

    loadLineEdit(parent->xBeamSourceRefPlaneText,parent->xBeamSourceRefPlane);
    loadLineEdit(parent->xBeamTargetRefPlaneText,parent->xBeamTargetRefPlane);
    loadLineEdit(parent->beamAngleText,parent->beamAngle);
    loadLineEdit(parent->ySnapText,parent->ySnap);
    loadLineEdit(parent->intersectionDotRadiusText,parent->intersectionDotRadius);
    loadLineEdit(parent->beamWidthText,parent->beamWidth);
    loadLineEdit(parent->interactionsText,parent->interactions);
    loadCheckBox(parent->divergingSourceCheckBox,parent->divergingSource);


}

void ControlPanelManager::readGlobalDisplay() {
    retrieveLineEdit(parent->yStartBeamPosText,parent->yStartBeamPos);
    retrieveLineEdit(parent->yEndBeamPosText,parent->yEndBeamPos);
    retrieveLineEdit(parent->dispersionBeamsText,parent->dispersionBeams);
    retrieveLineEdit(parent->tracingBeamsText,parent->tracingBeams);
    retrieveLineEdit(parent->xBeamSourceRefPlaneText,parent->xBeamSourceRefPlane);
    retrieveLineEdit(parent->xBeamTargetRefPlaneText,parent->xBeamTargetRefPlane);
    retrieveLineEdit(parent->beamAngleText,parent->beamAngle);
    retrieveLineEdit(parent->ySnapText,parent->ySnap);
    retrieveLineEdit(parent->intersectionDotRadiusText,parent->intersectionDotRadius);
    retrieveLineEdit(parent->beamWidthText,parent->beamWidth);
    retrieveLineEdit(parent->interactionsText,parent->interactions);
    retrieveCheckBox(parent->divergingSourceCheckBox,parent->divergingSource);
}
