/***************************************************************************
 *   Copyright (C) 2005 by Paul Lutus   *
 *   lutusp@pl-alpha   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LENS_H
#define LENS_H

#include <cmath>
#include <qpainter.h>
#include <box.h>
#include <iostream>
#include <sstream>
#include <string>
#include <raytracerwidgetcode.h>

using namespace std;

/**
@author Paul Lutus
*/
class Lens {
public:
    Lens(RayTracerWidgetCode *p,double cx,double cy,double lensRadius,double leftSphereRadius,double rightSphereRadius,double userThickness,double ior,double lcf,double rcf,double dispersion,bool leftHyp,bool rightHyp,bool symmetrical);

    Lens(RayTracerWidgetCode *p,string desc);

    ~Lens();

    string describe();
string explain();
    void setup();

    RayTracerWidgetCode *parent;

    double cx;
    double cy;
    double lensRadius;
    double leftOffset,rightOffset;
    double leftSphereRadius,rightSphereRadius;
    double userThickness,internalThickness;
    
    double ior;
    double lcf,rcf,dispersion;
    bool leftHyp,rightHyp,symmetrical;
    Box border;

    double  min(double x,double y);
    double  max(double x,double y);

    bool inside(double x,double y);
    void computeBoxSize(double y,Box &border,double &thick);

    void drawLens(QPainter &pixpainter);
    void drawBox(QPainter &pixpainter,bool focused);

    ComplexNum profile(int s);
    void lensXforY(double y,double ccx,double &xa,double &xb);
    double tangent(int profile,double y);

    void computeIntersections(int profile,double x1,double y1,double x2,double y2,double cx,double cy,double &xa,double &ya,double &xb,double &yb);


};

#endif
