/***************************************************************************
 *   Copyright (C) 2005 by Paul Lutus   *
 *   lutusp@pl-alpha   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef RAYTRACECOMPUTER_H
#define RAYTRACECOMPUTER_H

#include <colorpos.h>
#include <vector>
#include <qtabwidget.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qevent.h>

#include <cmath>
#include <string>

class Lens;

using namespace std;

/**
@author Paul Lutus
*/
class RayTraceComputer {
public:
    RayTraceComputer();

    ~RayTraceComputer();

    // BEGIN block of view and lens defining variables,
    // some to be saved and restored

    double dispScale,xOffset,yOffset;
    double lensMinThickness;

    double mousePressX,mousePressY;
    Lens *mouseTarget;
    Lens *boxTarget;
    int widgetOffsetX,widgetOffsetY;
    double beamWidth;

    double xSize,ySize,xCenter,yCenter;
    bool controlPanelOpen;

    long gridColor,yBaselineColor,dispColor,beamColor,lensDefColor,lensSelColor,lensOutlineColor,intersectionColor;

    double yStartBeamPos;
    double yEndBeamPos;
    double ySnap;
    double dispersionBeams;
    double tracingBeams;
    double intersectionDotRadius;
    double interactions;

    double xBeamSourceRefPlane;
    double xBeamTargetRefPlane;
    double beamAngle;
    bool divergingSource;


    int currentTab;

    // END block of defining variables

    static vector<string> split(string v,char delim);

    static const ColorPos colors[];

    static ColorPos createColorForWvl(double w);

    static const double dispersionPivot,dispersionFactor;

    static const double redWvlNm,vioWvlNm;

    static double dispersionIndex(double ior,double wavelength, double abbe);

    double colorForWavelength(double w);

    static const double toRadians,toDegrees;

    static void circle_line_intersections(
        double x1,
        double y1,
        double x2,
        double y2,
        double r,
        double cx,
        double cy,
        double &xa,
        double &ya,
        double &xb,
        double &yb);

    static void hyp_line_intersections(
        double x1,
        double y1,
        double x2,
        double y2,
        double cf,
        double r,
        double d,
        double cx,
        double cy,
        double &xa,
        double &ya,
        double &xb,
        double &yb);



    static double line_line_intersection(
        double ax1,double ay1,double ax2,double ay2,double bx1,double by1,double bx2,double by2
    );

    static double y_intercept(
        double x1, double y1, double x2, double y2, double xi
    );

    static double hyperbolicLensProfileXforY (double y,double cx,double cf, double sphereRadius, double offset,double thickness);

static double hyperbolicLensProfileDYforY (double y,double cx,double cf, double sphereRadius, double offset,double thickness);

    static double sphericalLensProfileXforY(double y, double cx, double sphereRadius, double offset,double thickness);
static double sphericalLensProfileDYforY(double y, double cx, double sphereRadius, double offset,double thickness);

    static double ntrp(double x,double xa, double xb, double ya, double yb);

    void spaceToDisplay(double x, double y,double &dx,double &dy);

    void displayToSpaceOffset(double dx, double dy,double &x,double &y);
    void displayToSpace(double dx, double dy,double &x,double &y);

    void scalePoint(double x, double y,int &ix,int &iy);

    void drawScaledLine(double x, double y,int &opx,int &opy, QPainter &pixpainter,QColor col,double width,bool draw);
    void drawScaledPoint(double x, double y,double radius, QPainter &pixpainter,QColor col);
    //void drawScaledEllipse(double x, double y,double radius, QPainter &pixpainter);



};

#endif
