/***************************************************************************
 *   Copyright (C) 2008 by Paul Lutus   *
 *   lutusp@pl-alpha   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "raytracerwidgetcode.h"
#include "controlpanelmanager.h"
#include "traceanddraw.h"
#include "lens.h"
#include "OpticalRayTracer.xpm"

const string RayTracerWidgetCode::programName = "OpticalRayTracer";
const string RayTracerWidgetCode::programVersion = "1.2";

RayTracerWidgetCode::RayTracerWidgetCode(QWidget* parent, const char* name, WFlags fl)
        : raytracerWidgetBase(parent,name,fl),RayTraceComputer() {

    usrPath = getenv("HOME");
    progPath = usrPath + "/." + programName;
    defaultConfigName = programName + ".ini";

    loadAboutBox();
    setIcon(QPixmap(OpticalRayTracer));

    // BEGIN default initializations

    dispScale = 0.2;
    xOffset = 2;
    yOffset = 0;
    currentTab = 0;
    dispColor = 0xffffff;
    beamColor = 0xff0000;
    lensSelColor = 0xe6ffe6;
    lensDefColor = 0xf0f0f0;
    intersectionColor = 0xaa00aa;
    intersectionDotRadius = 4;
    lensOutlineColor = 0x000000;
    yBaselineColor = 0x0000ff;
    gridColor = 0xa0ffff;

    yStartBeamPos = -1.8;
    yEndBeamPos = 1.8;
    dispersionBeams = 0;
    tracingBeams = 6;
    interactions = 64;

    xBeamSourceRefPlane = -30.0;
    xBeamTargetRefPlane = 0.0;
    ySnap = 1.0;
    beamAngle = 0;
    beamWidth = 0; // fast-draw line of width 1

    divergingSource = false;
    controlPanelOpen = true;

    // END default initializations

    lensMinThickness = .025; // % of lens radius

    overlappedLensSelector = 0;
    xSize = -1;
    ySize = -1;
    widgetOffsetX = -1;
    widgetOffsetY = -1;
    mouseTarget = NULL;
    boxTarget = NULL;
    colorSelectBusy = false;
    helpPageLayout = new QGridLayout( helpPage, 1, 1);

    helpwidget = new HelpWidget(helpPage,this);
    helpPageLayout->addWidget(helpwidget, 0, 0 );

    cpm = new ControlPanelManager(this);
    tad = new TraceAndDraw(this);

    //helpwidget->setMouseTracking(false);
    mainTabWidget->setMouseTracking(false);
    //graphicDisplay->grabMouse();

    setMouseTracking(true);

    initHandler = new InitFileHandler(usrPath,programName);
    readConfigFile(defaultConfigName);

    cpm->writeGlobalDisplay();
    setAllButtonColors();
    mainTabWidget->setCurrentPage(currentTab);

    first = true;

    // prevent system erasing your display
    graphicDisplay->setBackgroundMode(QWidget::NoBackground);

    // initial setting for control panel visibility
    controlPanelOpen = !controlPanelOpen;
    showHideControlPanel();
    setButtons();

    //graphicDisplay->setWFlags (Qt::WNoAutoErase);

}


RayTracerWidgetCode::~RayTracerWidgetCode() {

    delete(initHandler);
    delete(cpm);
    delete(tad);
}

void RayTracerWidgetCode::loadAboutBox() {

    string title = programName + " " + programVersion;
    setCaption(title);
    aboutGroupBox->setTitle((string) "About " + title);
    string data = title + " is Copyright (C) 2008 by Paul Lutus.\n\n"
                  + "For more information, additional documentation and the latest version,\n"
                  + "visit http://www.arachnoid.com/OpticalRayTracer.\n\n"
                  + "Also, if you are interested, visit http://www.arachnoid.com/careware.\n\n"
                  + "This program is free software; you can redistribute it and/or modify\n"
                  + "it under the terms of the GNU General Public License as published by\n"
                  + "the Free Software Foundation; either version 2 of the License, or\n"
                  + "(at your option) any later version.\n\n"
                  + "This program is distributed in the hope that it will be useful,\n"
                  + "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
                  + "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
                  + "GNU General Public License for more details.\n\n"
                  + "You should have received a copy of the GNU General Public License\n"
                  + "along with this program; if not, write to the\n"
                  + "Free Software Foundation, Inc.,\n"
                  + "59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n";
    aboutText->setText(data);
}

void RayTracerWidgetCode::windowActivationChange ( bool oldActive ) {
    update();
}

void RayTracerWidgetCode::resizeEvent ( const QSize & size, const QSize & oldSize ) {
    update();
}

void RayTracerWidgetCode::showEvent ( QShowEvent *e ) {
    if(first && e->type() == QEvent::Show) {
        tad->rayTraceProcess();
        first = false;
    }
}

void RayTracerWidgetCode::paintEvent ( QPaintEvent *p ) {
    //static int i = 0;
    //printf("paint event %d\n",i++);
    //fflush(stdout);
    tad->updateBuffer(false);
    if(tad->qpm != NULL && graphicDisplay != NULL) {
        bitBlt(graphicDisplay,0,0,tad->qpm);
    }
}

// must update on tab change
// otherwise no display when widget comes into view

void RayTracerWidgetCode::tabChanged(QWidget *w) {
    update();
}


void RayTracerWidgetCode::getWidgetOffset(QObject *p,int &x,int &y) {
    x = 0;
    y = 0;
    while(p->parent() != NULL) {
        x += ((QWidget *)p)->x();
        y += ((QWidget *)p)->y();
        p = p->parent();
    }
}

void RayTracerWidgetCode::mousePressEvent ( QMouseEvent * e ) {
    if(graphicDisplay->hasMouse()) {
        setCursor(QCursor(SizeAllCursor));
        int mx = e->x() - widgetOffsetX;
        int my = e->y() - widgetOffsetY;
        //printf("press event mx = %d, my = %d\n",mx,my);
        //fflush(stdout);
        double dx,dy;
        displayToSpace(mx,my,dx,dy);


        Lens *p = testMouseInsideLens(dx+xOffset,dy+yOffset);

        if(p == NULL) {
            mousePressX = dx + xOffset;
            mousePressY = dy + yOffset;
        } else {
            mousePressX = -dx + p->cx;
            mousePressY = -dy + p->cy;
            cpm->updateLensDisplay(p);
            if(boxTarget != p) {
                boxTarget = p;
                tad->rayTraceProcess(true);
            }
        }
        mouseTarget = p;
        tad->updateStatusLabel(mx,my,graphicDisplay->hasMouse());
    }
}

Lens *RayTracerWidgetCode::testMouseInsideLens(double mx, double my) {
    VL lensSet;
    VLI vli = tad->lensVec.begin();
    while(vli < tad->lensVec.end()) {
        if((*vli)->inside(mx,my)) {
            lensSet.push_back(*vli);
        }
        vli++;
    }
    if(lensSet.size() == 0) {
        return NULL;
    } else {
        // cycle between the overlapped selections
        // as the user presses the mouse repeatedly
        int i = overlappedLensSelector++ % lensSet.size();
        return lensSet[i];
    }
}

void RayTracerWidgetCode::snapLens(Lens *lens) {
    if(fabs(lens->cy) <= ySnap) {
        lens->cy = 0;
    }
}

void RayTracerWidgetCode::mouseReleaseEvent ( QMouseEvent * e ) {
    setCursor(QCursor(ArrowCursor));
    if(mouseTarget != NULL) {
        snapLens(mouseTarget);
        cpm->updateLensDisplay(mouseTarget);
    }
    int mx = e->x() - widgetOffsetX;
    int my = e->y() - widgetOffsetY;
    tad->rayTraceProcess(true);
    tad->updateStatusLabel(mx,my,graphicDisplay->hasMouse());
}

void RayTracerWidgetCode::mouseMoveEvent ( QMouseEvent * e ) {
    int mx = e->x() - widgetOffsetX;
    int my = e->y() - widgetOffsetY;
    if(e->state() == Qt::LeftButton && graphicDisplay->hasMouse()) {
        //printf("mx = %d, my = %d\n",mx,my);
        //fflush(stdout);
        double dx,dy;
        displayToSpace(mx,my,dx,dy);
        if(mouseTarget == NULL) {
            xOffset = -dx + mousePressX;
            yOffset = -dy + mousePressY;
        } else {
            mouseTarget->cx = dx + mousePressX;
            mouseTarget->cy = dy + mousePressY;
            cpm->updateLensDisplay(mouseTarget);
        }
        tad->rayTraceProcess(true);
    }
    tad->updateStatusLabel(mx,my,graphicDisplay->hasMouse());
    //printf("move event mx = %d, my = %d\n",mx,my);
    //fflush(stdout);
}

void RayTracerWidgetCode::wheelEvent ( QWheelEvent * e ) {
    int mx = e->x() - widgetOffsetX;
    int my = e->y() - widgetOffsetY;
    if(graphicDisplay->hasMouse()) {
        double f = (e->delta() > 0)?1.2:.8333;
        dispScale *= f;
        tad->rayTraceProcess(true);
    }
    tad->updateStatusLabel(mx,my,graphicDisplay->hasMouse());
    //printf("wheel event mx = %d, my = %d\n",mx,my);
    //fflush(stdout);
}

// reconfiguring this display turned out to be
// easier than I expected based on past experience

void RayTracerWidgetCode::showHideControlPanel() {
    controlPanelOpen = !controlPanelOpen;
    if(controlPanelOpen) {
        controlPanelGroup->setMaximumSize(32767,32767);
        showHideButton->setText("Hide Controls");
    } else {
        controlPanelGroup->setMaximumSize(0,0);
        showHideButton->setText("Show Controls");
        boxTarget = NULL;
        mouseTarget = NULL;
        cpm->enableControlPanel(false);
    }
    update();
}

void RayTracerWidgetCode::setButtons() {
    demoButton->setEnabled(tad->lensVec.size() == 0);
}

// must have this to prevent abnormal suhutdown

void RayTracerWidgetCode::closeEvent ( QCloseEvent * e ) {
    writeConfigFile(defaultConfigName);
    e->accept();
}

void RayTracerWidgetCode::readConfigFile(string name,bool full) {

    bool exists = initHandler->readConfigFile(name);
    if(exists) {
        if (full) {
            // app size and location
            int xv = x();
            int yv = y();
            initHandler->readValue("PosX",xv);
            initHandler->readValue("PosY",yv);
            move(xv,yv);

            xv = width();
            yv = height();
            initHandler->readValue("SizeX",xv);
            initHandler->readValue("SizeY",yv);
            resize(xv,yv);
        }

        // program values

        initHandler->readValue("lensOutlineColor",lensOutlineColor);
        initHandler->readValue("dispColor",dispColor);
        initHandler->readValue("beamColor",beamColor);
        initHandler->readValue("lensDefColor",lensDefColor);
        initHandler->readValue("lensSelColor",lensSelColor);
        initHandler->readValue("intersectionColor",intersectionColor);
        initHandler->readValue("gridColor",gridColor);

        initHandler->readValue("dispScale",dispScale);
        initHandler->readValue("xOffset",xOffset);
        initHandler->readValue("yOffset",yOffset);
        initHandler->readValue("controlPanelOpen",controlPanelOpen);
        initHandler->readValue("currentTab",currentTab);
        initHandler->readValue("yStartBeamPos",yStartBeamPos);
        initHandler->readValue("yEndBeamPos",yEndBeamPos);
        initHandler->readValue("dispersionBeams",dispersionBeams);
        initHandler->readValue("tracingBeams",tracingBeams);
        initHandler->readValue("ySnap",ySnap);
        initHandler->readValue("beamWidth",beamWidth);
        initHandler->readValue("beamAngle",beamAngle);
        initHandler->readValue("intersectionDotRadius",intersectionDotRadius);
        initHandler->readValue("interactions",interactions);
        initHandler->readValue("divergingSource",divergingSource);

        initHandler->readValue("lensTerseDescriptions",lensTerseDescriptions);
        decodeLensDescriptions();
    } else {
        tad->makeDefaultLenses();
    }
}

void RayTracerWidgetCode::writeConfigFile(string name,bool full) {

    if(full) {
        // app size and location
        initHandler->writeValue("PosX",x());
        initHandler->writeValue("PosY",y());
        initHandler->writeValue("SizeX",width());
        initHandler->writeValue("SizeY",height());
    }

    // program values
    initHandler->writeValue("lensOutlineColor",lensOutlineColor);
    initHandler->writeValue("yBaselineColor",yBaselineColor);
    initHandler->writeValue("dispColor",dispColor);
    initHandler->writeValue("beamColor",beamColor);
    initHandler->writeValue("lensDefColor",lensDefColor);
    initHandler->writeValue("lensSelColor",lensSelColor);
    initHandler->writeValue("intersectionColor",intersectionColor);
    initHandler->writeValue("gridColor",gridColor);
    currentTab = mainTabWidget->currentPageIndex();
    initHandler->writeValue("dispScale",dispScale);
    initHandler->writeValue("xOffset",xOffset);
    initHandler->writeValue("yOffset",yOffset);
    initHandler->writeValue("controlPanelOpen",controlPanelOpen);
    initHandler->writeValue("currentTab",currentTab);

    initHandler->writeValue("yStartBeamPos",yStartBeamPos);
    initHandler->writeValue("yEndBeamPos",yEndBeamPos);
    initHandler->writeValue("dispersionBeams",dispersionBeams);
    initHandler->writeValue("tracingBeams",tracingBeams);
    initHandler->writeValue("ySnap",ySnap);
    initHandler->writeValue("beamWidth",beamWidth);
    initHandler->writeValue("beamAngle",beamAngle);
    initHandler->writeValue("intersectionDotRadius",intersectionDotRadius);
    initHandler->writeValue("interactions",interactions);
    initHandler->writeValue("divergingSource",divergingSource);

    encodeLensDescriptions();
    initHandler->writeValue("lensTerseDescriptions",lensTerseDescriptions);

    initHandler->writeConfigFile(name);
    addLensDetails(name);

}

void RayTracerWidgetCode::decodeLensDescriptions() {
    vector<string> vs = split(lensTerseDescriptions,'|');
    vector<string>::iterator vsi = vs.begin();
    while(vsi != vs.end()) {
        if((*vsi).length() > 0) {
            tad->lensVec.push_back(new Lens(this,(*vsi)));
        }
        vsi++;
    }
}

void RayTracerWidgetCode::encodeLensDescriptions() {
    stringstream ss;
    VLI vli = tad->lensVec.begin();
    string delim="";
    while(vli != tad->lensVec.end()) {
        ss << delim + (*vli)->describe();
        delim="|";
        vli++;
    }
    lensTerseDescriptions = ss.str();
}

void RayTracerWidgetCode::addLensDetails(string name) {
    if(tad->lensVec.size() > 0) {
        string path = initHandler->appPath + "/" + name;
        ofstream ofs(path.c_str(),ios::app);
        VLI vli = tad->lensVec.begin();
        int n = 1;
        ofs << "\n# Human-readable Lens Descriptions:\n";
        while(vli != tad->lensVec.end()) {
            ofs << "\n# *** Lens " << n << " ***\n";
            ofs << (*vli)->explain();
            vli++;
            n++;
        }
        ofs.close();
    }
}

void RayTracerWidgetCode::selectColor(long &color,QPushButton *b) {
    b->setOn(false);
    QColor col = QColorDialog::getColor(QColor(color));
    if(col.isValid()) {
        color = col.rgb();
    }
    setButtonColor(b,color);

}

void RayTracerWidgetCode::setAllButtonColors() {
    setButtonColor(dispColorButton,dispColor);
    setButtonColor(beamColorButton,beamColor);
    setButtonColor(lensDefColorButton,lensDefColor);
    setButtonColor(lensSelColorButton,lensSelColor);
    setButtonColor(intersectionColorButton,intersectionColor);
    setButtonColor(lensOutlineColorButton,lensOutlineColor);
    setButtonColor(yBaselineColorButton,yBaselineColor);
    setButtonColor(gridColorButton,gridColor);
}

void RayTracerWidgetCode::setButtonColor(QPushButton *b,long color) {
    b->setPaletteBackgroundColor(QColor(color));
}

void RayTracerWidgetCode::lensDescriptionChanged() {
    cpm->changeLens();
}

void RayTracerWidgetCode::deleteLens() {
    cpm->deleteLens();
    setButtons();
}


void RayTracerWidgetCode::cloneLens() {
    cpm->cloneLens();
}


void RayTracerWidgetCode::newLens() {
    cpm->newLens();
    setButtons();
}

void RayTracerWidgetCode::showHidePressed() {
    showHideControlPanel();
}

void RayTracerWidgetCode::chooseColor() {
    if(!colorSelectBusy) {
        bool valid = true;
        colorSelectBusy = true;
        if(dispColorButton->isOn()) {
            selectColor(dispColor,dispColorButton);
        } else if(beamColorButton->isOn()) {
            selectColor(beamColor,beamColorButton);
        } else if(lensDefColorButton->isOn()) {
            selectColor(lensDefColor,lensDefColorButton);
        } else if(lensSelColorButton->isOn()) {
            selectColor(lensSelColor,lensSelColorButton);
        } else if(intersectionColorButton->isOn()) {
            selectColor(intersectionColor,intersectionColorButton);
        } else if(lensOutlineColorButton->isOn()) {
            selectColor(lensOutlineColor,lensOutlineColorButton);
        } else if(yBaselineColorButton->isOn()) {
            selectColor(yBaselineColor,yBaselineColorButton);
        } else if(gridColorButton->isOn()) {
            selectColor(gridColor,gridColorButton);
        } else {
            valid = false;
            //printf("can't find button!\n");
            //fflush(stdout);
        }
        if(valid) {
            tad->rayTraceProcess();
        }
        colorSelectBusy = false;
    }
}

void RayTracerWidgetCode::globalDescriptionChanged() {
    cpm->readGlobalDisplay();
    cpm->writeGlobalDisplay();
    tad->rayTraceProcess();
}

void RayTracerWidgetCode::demoLenses() {
    tad->makeDefaultLenses();
    setButtons();
}






