/***************************************************************************
 *   Copyright (C) 2005 by Paul Lutus   *
 *   lutusp@pl-alpha   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef RAYTRACERWIDGETCODE_H
#define RAYTRACERWIDGETCODE_H

#include "helpwidget.h"
#include "raylensintersection.h"
#include "raytracerwidgetbase.h"
#include "initfilehandler.h"
#include "raytracecomputer.h"

#include <qcursor.h>
#include <qcolordialog.h>
#include <qtabwidget.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qevent.h>
#include <qlayout.h>

#include <vector>
#include <cmath>
#include <string>
#include <fstream>
#include <algorithm>

class ControlPanelManager;
class DisplayPane;
class TraceAndDraw;
class Lens;


using namespace std;

/**
@author Paul Lutus
*/
class RayTracerWidgetCode : public raytracerWidgetBase, public RayTraceComputer {

public:
    RayTracerWidgetCode(QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );

    ~RayTracerWidgetCode();

void loadAboutBox();

    QGridLayout *helpPageLayout;

    HelpWidget *helpwidget;

    void paintEvent ( QPaintEvent *p );
    void resizeEvent ( const QSize & size, const QSize & oldSize );
    void windowActivationChange ( bool oldActive );


    void tabChanged(QWidget *w);

    TraceAndDraw *tad;

    ControlPanelManager *cpm;
    //DisplayPane *dp;

    InitFileHandler *initHandler;

    void closeEvent ( QCloseEvent * e );

    Lens *testMouseInsideLens(double mx, double my);
void snapLens(Lens *lens);
    void mousePressEvent ( QMouseEvent * e );
    void mouseReleaseEvent ( QMouseEvent * e );
    void mouseMoveEvent ( QMouseEvent * e );
    void wheelEvent ( QWheelEvent * e );
    void showEvent ( QShowEvent * );
    void showHideControlPanel();
void setButtons();

    static const string programName;
    static const string programVersion;

    string usrPath,progPath,defaultConfigName,lensTerseDescriptions;

    bool first;

int overlappedLensSelector;

    void readConfigFile(string name,bool full = true);
    void writeConfigFile(string name,bool full = true);
    void getWidgetOffset(QObject *p,int &x,int &y);

    void decodeLensDescriptions();
    void encodeLensDescriptions();
void addLensDetails(string name);
    void selectColor(long &color,QPushButton *b);
    void setAllButtonColors();
    void setButtonColor(QPushButton *b,long color);
    bool colorSelectBusy;

protected:
    /*$PROTECTED_FUNCTIONS$*/

public slots:
    virtual void deleteLens();
public slots:
    virtual void cloneLens();
    virtual void globalDescriptionChanged();
public slots:
    virtual void newLens();
public slots:
    virtual void lensDescriptionChanged();
    virtual void showHidePressed();
    virtual void chooseColor();
    virtual void demoLenses();
};

#endif
